from colour_printing.markers import Markers
from colour_printing.dyestuff import Dyestuff

default_info = "info"
default_warn = "warn"
default_success = "success"
default_error = "error"


class ColourPrint:
    Markers = Markers

    def __init__(self):
        self.__default_setting()

    def __init_subclass__(cls, **kwargs):
        cls.custom(cls)

    @staticmethod
    def dyestuff(*args, mode='', fore='', back=''):
        return Dyestuff.dyestuff(*args, mode=mode, fore=fore, back=back)

    def custom(self):
        e = '''
        Custom styles need to override this method ! 
        for example : 
            self.debug = self.Markers('debug').flag_style(mode="bold").time_style(...)'''
        raise NotImplementedError(e)

    def __default_setting(self):
        self.info = Markers(default_info) \
            .flag_style(mode='bold', fore='blue').time_style(mode='invert').message_style(mode='bold', fore='blue')

        self.warn = Markers(default_warn) \
            .flag_style(mode='bold', fore='yellow').time_style(mode='invert').message_style(mode='bold', fore='yellow')

        self.success = Markers(default_success) \
            .flag_style(mode='bold', fore='green').time_style(mode='invert').message_style(mode='bold', fore='green')

        self.error = Markers(default_error) \
            .flag_style(mode='bold', fore='red').time_style(mode='invert').message_style(mode='bold', fore='red')

    def __str__(self):
        return """
                @'fore': # 前景色
                        'black': 黑色
                        'red': 红色
                        'green': 绿色
                        'yellow': 黄色
                        'blue':  蓝色
                        'purple':  紫红色
                        'cyan':  青蓝色
                        'white':  白色
                @'back':# 背景
                        'black':  黑色
                        'red':  红色
                        'green': 绿色
                        'yellow':  黄色
                        'blue': 蓝色
                        'purple':  紫红色
                        'cyan':  青蓝色
                        'white': 白色
                @'mode':# 显示模式
                        'normal': 终端默认设置
                        'bold':  高亮显示
                        'underline':  使用下划线
                        'blink': 闪烁
                        'invert': 反白显示
                        'hide': 不可见
            """



from colour_printing import ColourPrint


class Log(ColourPrint):
    def custom(self):
        self.debug = self.Markers('debug').flag_style(mode='hide').time_style(mode='hide').message_style(fore='red')


log = Log()
print(log)
log.debug('123', 'sda', 'sadasd')

s1 = log.dyestuff('this red', fore='red')
s2 = Log.dyestuff('this green', fore='green')
s3 = ColourPrint.dyestuff('this blue', fore='blue')
print(s1, s2, s3)



from datetime import datetime
from colour_printing.style import setting
from colour_printing.switch import Switch

MESSAGE_STYLE = 'message_style'
FLAG_STYLE = 'flag_style'
TIME_STYLE = 'time_style'


class Markers:
    __get_time = lambda _: datetime.strftime(datetime.now(), '%Y-%m-%d %H:%M:%S')
    __template = '{flag_style0}{flag}{flag_style1}{time_style0}{time}{time_style1}' \
                 '{message_style0}{string}{message_style1}'
    __flag_len = 7

    def __init__(self, flag_name: str):
        self.__flag_name = flag_name.upper()
        self.__config = {}
        self.__cal_flag_len()
        self.__hide = []

    def __cal_flag_len(self):
        if Markers.__flag_len < len(self.__flag_name):
            Markers.__flag_len = len(self.__flag_name)

    def message_style(self, mode='', fore='', back=''):
        self.__config[MESSAGE_STYLE] = setting(mode=mode, fore=fore, back=back)
        return self

    def time_style(self, mode='', fore='', back=''):
        if mode == 'hide':
            self.__hide.append('time')
        else:
            self.__config[TIME_STYLE] = setting(mode=mode, fore=fore, back=back)
        return self

    def flag_style(self, mode='', fore='', back=''):
        if mode == 'hide':
            self.__hide.append('flag')
        else:
            self.__config[FLAG_STYLE] = setting(mode=mode, fore=fore, back=back)
        return self

    def __user_setting(self):
        message_style = self.__config.get(MESSAGE_STYLE, setting())
        time_style = self.__config.get(TIME_STYLE, setting())
        flag_style = self.__config.get(FLAG_STYLE, setting())
        return self.__template.format(flag_style0=flag_style[0],
                                      flag=f"[{self.__flag_name.center(self.__flag_len, '-')}] "
                                      if 'flag' not in self.__hide else "",
                                      flag_style1=flag_style[1],
                                      time_style0=time_style[0],
                                      time=f"{self.__get_time()} " if 'time' not in self.__hide else '',
                                      time_style1=time_style[1],
                                      message_style0=message_style[0],
                                      string='{}',
                                      message_style1=message_style[1])

    def __call__(self, *args):
        if not Switch.signal or self.__flag_name.lower() in Switch.filter or \
                self.__flag_name in Switch.filter:
            return
        template = self.__user_setting().replace('{}', len(args) * '{} ')
        print(template.format(*args))



from colour_printing.style import setting


class Dyestuff:
    @classmethod
    def dyestuff(cls, *args, **kwargs):
        """
        :param args: 字符
        :param kwargs: 颜色参数
        :return:  彩色字符
        """
        style = setting(**kwargs)
        temp = f'{style[0]}{("{} " * len(args))}{style[1]}'
        return temp.format(*args)



import setuptools
import codecs
import os


def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()


setuptools.setup(
    name="colour-printing",
    version="0.1.7",
    author="faithforus",
    author_email="ljunf817@163.com",
    description="colour-printing",
    long_description=read('README.rst'),
    keywords="python package print",
    url="https://github.com/Faithforus/Colour-printing",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
